/*
 *  Copyright 2009 Peter Karich.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core.ui.planner;

import de.timefinder.algo.DataPoolSettings;
import de.timefinder.data.Event;
import java.util.ArrayList;
import java.util.Collection;
import javolution.util.FastMap;
import java.util.List;
import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JLabel;

/**
 * This component lists all added events in an HTML table.
 *
 * @author Peter Karich
 */
public class HtmlPlannerComponent {

    private List<Event> events;
    private DataPoolSettings settings;

    public HtmlPlannerComponent(DataPoolSettings settings) {
        this.settings = settings;
        init();
    }

    /**
     * This method add the events. The underlying implementation
     * should handle each added events collection as it comes from a
     * different resource. (one resource per method call)
     */
    public void addEvents(Collection<Event> events) {
        this.events.addAll(events);
    }

    public void init() {
        events = new ArrayList<Event>();
    }

    public JComponent createControl() {
        return new JLabel(getHtml(events));
    }

    private void printHeader(StringBuffer sb) {
        sb.append("<thead> <tr>");
        for (int day = 0; day < settings.getNumberOfDays(); day++) {
            sb.append("<th>");
            sb.append(getDayName(day));
            sb.append("</th>");
        }
        sb.append("</tr> </thead>");
    }

    private String getHtml(Collection<Event> events) {
        List<Event> notAssigned = new ArrayList<Event>();
        Map<Integer, List<Event>> map = new FastMap<Integer, List<Event>>();

        for (Event event : events) {

            if (false /*event.isAssigned()*/) {
                notAssigned.add(event);
                continue;
            }

            List<Event> list = map.get(event.getStart());
            if (list == null) {
                list = new ArrayList<Event>();
                map.put(event.getStart(), list);
            }
            list.add(event);
        }

        StringBuffer sb = new StringBuffer();
        sb.append("<html>");
        sb.append("<table  border=\"1\">");
        // header of table
        printHeader(sb);

        // body of table
        sb.append("<tbody>");
        for (int timeslot = 0; timeslot < settings.getTimeslotsPerDay(); timeslot++) {
            sb.append("<tr>");
            for (int day = 0; day < settings.getNumberOfDays(); day++) {
                sb.append("<td>");
                List<Event> list = map.get(timeslot + settings.getTimeslotsPerDay() * day);
                if (list == null) {
                    continue;
                }
                for (Event event : list) {
                    sb.append(toString(event));
                    sb.append("<br/>");
                }
                sb.append("</td>");
            }
            sb.append("</tr>");
        }
        sb.append("</tbody>");
        sb.append("</table>");

        // footer
        sb.append("<br/><br/>Unassigned events:<br/>");
        for (Event event : notAssigned) {
            sb.append("<div>");
            sb.append(toString(event));
            sb.append("</div>");
        }

        // end
        sb.append("</html>");
        //log.info(sb.toString());
        return sb.toString();
    }

    String getDayName(int i) {
        switch (i) {
            case 0:
                return "Mon";
            case 1:
                return "Tue";
            case 2:
                return "Wed";
            case 3:
                return "Thu";
            case 4:
                return "Fri";
            case 5:
                return "Sat"; 
            case 6:
                return "Fun";
            default:
                return "Day" + i;
        }
    }

    String toString(Event event) {
        String str = "from:" + event.getStart() + ", duration:" + event.getDuration() +
                ", " + event.getName() + " (" + event.getId() + ") <br />";
        if (event.getLocation() != null) {
            str += "location:" + event.getLocation().getName();
        }
        return str;
    }
}
