/*
 * This file is part of the TimeFinder project.
 *  Visit http://www.timefinder.de for more information.
 *  Copyright (c) 2009 the original author or authors.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package de.timefinder.core.util;

import de.pannous.xvantage.core.DataPool;
import de.pannous.xvantage.core.Xvantage;
import de.pannous.xvantage.core.impl.DefaultDataPool;
import de.timefinder.core.springrc.AbstractSettings;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import javolution.util.FastMap;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.core.io.Resource;
import org.springframework.richclient.settings.Settings;
import org.springframework.richclient.util.Assert;

import java.net.MalformedURLException;
import java.security.AccessControlException;
import java.util.Map;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

/**
 * @author Peter Karich, peat hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class XmlSettings extends AbstractSettings {

    private static final String couldnotLoad = "Couldn't load object from file - ";
    private static final String zipEntryName = "xml";
    private File file;
    private Log log = LogFactory.getLog(getClass());
    private FastMap<String, Object> settingsMap;

    public XmlSettings(Settings parent, String name) {
        this(parent, name, null);
    }

    public XmlSettings(Settings parent, String name, File file) {
        super(parent, name);
        setLocation(file);
        settingsMap = new FastMap<String, Object>();
    }

    /**
     * User can either use location or file to specify the source of this
     * settings.
     */
    public void setLocation(Resource location) {
        try {
            file = location.getFile();
        } catch (IOException ex) {
            log.info("Can't get file from location.", ex);
        }
    }

    private File getFile() {
        return file;
    }

    public void setLocation(File file) {
        this.file = file;
    }

    private Xvantage newXvantage() {
        Xvantage xvantage = new Xvantage();
        xvantage.mount("/settings/map", FastMap.class);
        xvantage.setDefaultImplementation(Map.class, FastMap.class);
        xvantage.setSkipNullProperty(true);
        return xvantage;
    }

    public void save() throws IOException {
        OutputStream os;
        File xmlFile = getFile();
        if (xmlFile == null) {
            log.info("Couldn't save settings - File was null.");
            return;
        }
        try {
            os = new FileOutputStream(xmlFile);
            try {
                if (xmlFile.getName().endsWith(".zip")) {
                    ZipOutputStream zip = new ZipOutputStream(os);
                    zip.putNextEntry(new ZipEntry(zipEntryName));
                    os = zip;
                } else if (xmlFile.getName().endsWith(".gz")) {
                    GZIPOutputStream zip = new GZIPOutputStream(os);
                    os = zip;
                }
//                new XStream().toXML(settingsMap, os);
                Xvantage xvantage = newXvantage();
                DataPool pool = new DefaultDataPool();
                pool.getData(FastMap.class).put(0L, settingsMap);
                xvantage.saveObjects(new OutputStreamWriter(os), pool);
                log.info("Saved object to:" + " " + xmlFile.getAbsolutePath());
            } finally {
                os.close();
            }
        } catch (MalformedURLException e) {
            log.warn("Malformed URL:", e);
        } catch (FileNotFoundException ex) {
            log.warn("Can't find file!", ex);
        } catch (IOException e) {
            log.warn("Can't gzip the file: IOException:", e);
        }
    }

    public void load() throws IOException {
        File xmlFile = getFile();
        if (xmlFile == null) {
            log.info(couldnotLoad + "File was null.");
            return;
        }
        try {
            InputStream is = new FileInputStream(xmlFile);
            try {
                if (xmlFile.getName().endsWith(".gz")) {
                    is = new GZIPInputStream(is);
                } else if (xmlFile.getName().endsWith(".zip")) {
                    is = new ZipFile(xmlFile).getInputStream(new ZipEntry(zipEntryName));
                }
                Xvantage xvantage = newXvantage();
                DataPool pool = xvantage.readObjects(is);
                settingsMap = pool.getData(FastMap.class).get(0L);

//                settingsMap = (Map<String, Object>) new XStream().fromXML(is);
                log.info("Successfully readed: " + xmlFile.getAbsolutePath());
            } finally {
                is.close();
            }
        } catch (FileNotFoundException e) {
            log.warn(couldnotLoad + "File not found. " + e.getLocalizedMessage());
        } catch (IOException e) {
            log.warn(couldnotLoad + "May be the gzipped file is currupt or it is the wrong file format.", e);
        } catch (AccessControlException e) {
            log.warn(couldnotLoad +
                    "Please specify this in your security policy file!", e);
        } catch (Exception e) {
            log.warn(couldnotLoad +
                    "May be it isn't the correct file version!", e);
        }
    }

    @Override
    protected boolean internalContains(String key) {
        return settingsMap.containsKey(key);
    }

    @Override
    protected String[] internalGetChildSettings() {
        return new String[]{};
    }

    @Override
    protected Settings internalCreateChild(String key) {
        throw new UnsupportedOperationException("Unsupported operation");
    }

    @Override
    protected void internalRemoveSettings() {
        throw new UnsupportedOperationException("Unsupported operation");
    }

    @Override
    protected void internalSet(String key, String value) {
        settingsMap.put(key, value);
    }

    @Override
    protected String internalGet(String key) {
        Object o = settingsMap.get(key);
        if (o == null) {
            return null;
        }
        return o.toString();
    }

    @Override
    protected void internalRemove(String key) {
        settingsMap.remove(key);
    }

    public String[] getKeys() {
        return (String[]) settingsMap.keySet().toArray();
    }

    public void setObject(String key, Object newValue) {
        Assert.notNull(key, "Key cannot be null");

        Object old = getObject(key);
        settingsMap.put(key, newValue);
        afterSet(key, old, newValue);
    }

    public Object getObject(String key) {
        Assert.notNull(key, "Key cannot be null");

        Object value = settingsMap.get(key);
        if (value == null) {
            return getDefault(key);
        }
        return value;
    }

    public void setDefault(String key, Object value) {
        Assert.notNull(key, "Key cannot be null");

        defaults.put(key, value);
    }

    public Object getDefault(String key) {
        Assert.notNull(key, "Key cannot be null");

        return defaults.get(key);
    }
}
